@@ -41,9 +41,11 @@ void AbstractFileListener::setDatabaseInterface(DatabaseInterface *model)
 {
     if (model) {
         connect(this, &AbstractFileListener::newTrackFile, d->mFileListing, &AbstractFileListing::newTrackFile);
-        connect(d->mFileListing, &AbstractFileListing::tracksList, model, &DatabaseInterface::insertTracksList);
+        connect(d->mFileListing, &AbstractFileListing::tracksList,
+                model, &DatabaseInterface::insertTracksList);
         connect(d->mFileListing, &AbstractFileListing::removedTracksList, model, &DatabaseInterface::removeTracksList);
-        connect(d->mFileListing, &AbstractFileListing::modifyTracksList, model, &DatabaseInterface::insertTracksList);
+        connect(d->mFileListing, &AbstractFileListing::modifyTracksList,
+                model, &DatabaseInterface::insertTracksList);
         connect(d->mFileListing, &AbstractFileListing::askRestoredTracks,
                 model, &DatabaseInterface::askRestoredTracks);
         connect(model, &DatabaseInterface::restoredTracks,
