@@ -13,6 +13,7 @@
 #include "datatypes.h"
 
 #include <QObject>
+#include <QQmlEngine>
 #include <QString>
 #include <QHash>
 #include <QList>
@@ -30,6 +31,10 @@ class ELISALIB_EXPORT DatabaseInterface : public QObject
 {
     Q_OBJECT
 
+    QML_ELEMENT
+
+    QML_UNCREATABLE("")
+
 public:
 
     enum DatabaseVersion {
@@ -85,6 +90,8 @@ public:
 
     DataTypes::ListTrackDataType tracksDataFromGenre(const QString &genre);
 
+    DataTypes::ListTrackDataType tracksDataFromGenreAndAuthor(const QString &genre, const QString &artistName);
+
     DataTypes::TrackDataType trackDataFromDatabaseId(qulonglong id);
 
     DataTypes::TrackDataType trackDataFromDatabaseIdAndUrl(qulonglong id, const QUrl &trackUrl);
@@ -101,27 +108,25 @@ public:
 
 Q_SIGNALS:
 
+    void tracksAdded(const DataTypes::ListTrackDataType &allTracks);
+    void radioAdded(const DataTypes::TrackDataType &radio);
+    void albumsAdded(const DataTypes::ListAlbumDataType &newAlbums);
     void artistsAdded(const DataTypes::ListArtistDataType &newArtists);
-
+    void genresAdded(const DataTypes::ListGenreDataType &NewGenres);
     void composersAdded(const DataTypes::ListArtistDataType &newComposers);
-
     void lyricistsAdded(const DataTypes::ListArtistDataType &newLyricists);
 
-    void albumsAdded(const DataTypes::ListAlbumDataType &newAlbums);
-
-    void tracksAdded(const DataTypes::ListTrackDataType &allTracks);
-
-    void genresAdded(const DataTypes::ListGenreDataType &allGenres);
-
-    void artistRemoved(qulonglong removedArtistId);
-
+    void trackRemoved(qulonglong removedTrackId);
+    void radioRemoved(qulonglong radioId);
     void albumRemoved(qulonglong removedAlbumId);
-
-    void trackRemoved(qulonglong id);
-
-    void albumModified(const DataTypes::AlbumDataType &modifiedAlbum, qulonglong modifiedAlbumId);
+    void artistRemoved(qulonglong removedArtistId);
+    void genreRemoved(qulonglong removedGenreId);
+    void composerRemoved(qulonglong removedArtistId);
+    void lyricistRemoved(qulonglong removedLyricistId);
 
     void trackModified(const DataTypes::TrackDataType &modifiedTrack);
+    void radioModified(const DataTypes::TrackDataType &radio);
+    void albumModified(const DataTypes::AlbumDataType &modifiedAlbum, qulonglong modifiedAlbumId);
 
     void requestsInitDone();
 
@@ -135,15 +140,9 @@ Q_SIGNALS:
 
     void finishRemovingTracksList();
 
-    void radioAdded(const DataTypes::TrackDataType &radio);
-
-    void radioModified(const DataTypes::TrackDataType &radio);
-
-    void radioRemoved(qulonglong radioId);
-
 public Q_SLOTS:
 
-    void insertTracksList(const DataTypes::ListTrackDataType &tracks, const QHash<QString, QUrl> &covers);
+    void insertTracksList(const DataTypes::ListTrackDataType &tracks);
 
     void removeTracksList(const QList<QUrl> &removedTracks);
 
@@ -224,6 +223,8 @@ private:
 
     void initChangesTrackers();
 
+    void emitTrackerChanges();
+
     void recordModifiedTrack(qulonglong trackId);
 
     void recordModifiedAlbum(qulonglong albumId);
@@ -249,6 +250,8 @@ private:
 
     DataTypes::ListTrackDataType internalTracksFromGenre(const QString &genre);
 
+    DataTypes::ListTrackDataType internalTracksFromAuthorAndGenre(const QString &artistName, const QString &genre);
+
     QList<qulonglong> internalAlbumIdsFromAuthor(const QString &artistName);
 
     qulonglong insertAlbum(const QString &title, const QString &albumArtist,
@@ -258,18 +261,27 @@ private:
                            const DataTypes::TrackDataType &currentTrack, const QString &albumPath);
 
     qulonglong insertArtist(const QString &name);
-
-    qulonglong internalArtistIdFromName(const QString &name);
-
     qulonglong insertGenre(const QString &name);
+    qulonglong insertComposer(const QString &name);
+    qulonglong insertLyricist(const QString &name);
 
-    void removeTrackInDatabase(qulonglong trackId);
+    /**
+     * @returns the database ID, or 0 if the item could not be found
+     */
+    qulonglong internalGenericIdFromName(QSqlQuery &query);
+    qulonglong internalArtistIdFromName(const QString &name);
+    qulonglong internalGenreIdFromName(const QString &name);
+    qulonglong internalComposerIdFromName(const QString &name);
+    qulonglong internalLyricistIdFromName(const QString &name);
 
     void updateTrackInDatabase(const DataTypes::TrackDataType &oneTrack, const QString &albumPath);
 
+    void removeTrackInDatabase(qulonglong trackId);
     void removeAlbumInDatabase(qulonglong albumId);
-
     void removeArtistInDatabase(qulonglong artistId);
+    void removeGenreInDatabase(qulonglong genreId);
+    void removeComposerInDatabase(qulonglong composerId);
+    void removeLyricistInDatabase(qulonglong lyricistId);
 
     void reloadExistingDatabase();
 
@@ -279,8 +291,7 @@ private:
 
     void updateTrackOrigin(const QUrl &fileName, const QDateTime &fileModifiedTime);
 
-    qulonglong internalInsertTrack(const DataTypes::TrackDataType &oneModifiedTrack,
-                                   const QHash<QString, QUrl> &covers, bool &isInserted);
+    qulonglong internalInsertTrack(const DataTypes::TrackDataType &oneModifiedTrack, bool &isInserted);
 
     [[nodiscard]] DataTypes::TrackDataType buildTrackDataFromDatabaseRecord(const QSqlRecord &trackRecord) const;
 
@@ -294,10 +305,6 @@ private:
 
     bool isValidArtist(qulonglong albumId);
 
-    qulonglong insertComposer(const QString &name);
-
-    qulonglong insertLyricist(const QString &name);
-
     QHash<QUrl, QDateTime> internalAllFileName();
 
     bool internalGenericPartialData(QSqlQuery &query);
@@ -309,8 +316,10 @@ private:
     DataTypes::ListTrackDataType internalOneAlbumData(qulonglong databaseId);
 
     DataTypes::AlbumDataType internalOneAlbumPartialData(qulonglong databaseId);
-
     DataTypes::ArtistDataType internalOneArtistPartialData(qulonglong databaseId);
+    DataTypes::GenreDataType internalOneGenrePartialData(qulonglong databaseId);
+    DataTypes::ArtistDataType internalOneLyricistPartialData(qulonglong databaseId);
+    DataTypes::ArtistDataType internalOneComposerPartialData(qulonglong databaseId);
 
     DataTypes::ListTrackDataType internalAllTracksPartialData();
 
@@ -343,10 +352,26 @@ private:
 
     void updateTrackFinishedStatistics(const QUrl &fileName, const QDateTime &time);
 
-    void internalInsertOneTrack(const DataTypes::TrackDataType &oneTrack, const QHash<QString, QUrl> &covers);
+    void internalInsertOneTrack(const DataTypes::TrackDataType &oneTrack);
 
     void internalInsertOneRadio(const DataTypes::TrackDataType &oneTrack);
 
+    /**
+     * Execute a query that checks whether there is at least 1 matching row
+     */
+    bool execHasRowQuery(QSqlQuery &query);
+
+    bool artistHasTracks(qulonglong artistId);
+    bool genreHasTracks(qulonglong genreId);
+    bool composerHasTracks(qulonglong composerId);
+    bool lyricistHasTracks(qulonglong lyricistId);
+
+    void pruneCollections();
+    void pruneArtists();
+    void pruneGenres();
+    void pruneComposers();
+    void pruneLyricists();
+
     std::unique_ptr<DatabaseInterfacePrivate> d;
 
 };
